@@ -73,6 +73,7 @@ SUB_PROGRESS = "qbzr:PROGRESS:"
 SUB_GETPASS = "qbzr:GETPASS:"
 SUB_GETUSER = "qbzr:GETUSER:"
 SUB_GETBOOL = "qbzr:GETBOOL:"
+SUB_CHOOSE = "qbzr:CHOOSE:"
 SUB_ERROR = "qbzr:ERROR:"
 
 
@@ -484,6 +485,7 @@ class SubProcessWidget(QtGui.QWidget):
         if hide_progress:
             self.hide_progress()
 
+        self.force_passing_args_via_file = False
         self._args_file = None  # temp file to pass arguments to qsubprocess
         self.error_class = ''
         self.error_data = {}
@@ -549,17 +551,19 @@ class SubProcessWidget(QtGui.QWidget):
             or re.search(r"(?:"
                 r"\n|\r"            # workaround for bug #517420
                 r"|\\\\"            # workaround for bug #528944
-                r")", args) is not None):
+                r")", args) is not None
+            or self.force_passing_args_via_file     # workaround for bug #936587
+            ):
             # save the args to the file
             fname = self._create_args_file(args)
             args = "@" + fname.replace('\\', '/')
 
         if dir is None:
             dir = self.defaultWorkingDir
-            
+
         self.error_class = ''
         self.error_data = {}
-        
+
         self.process.setWorkingDirectory(dir)
         if getattr(sys, "frozen", None) is not None:
             bzr_exe = sys.executable
@@ -621,10 +625,7 @@ class SubProcessWidget(QtGui.QWidget):
         if progress == 1000000 and not messages:
             text = gettext("Finished!")
         else:
-            if isinstance(messages, unicode):
-                text = messages
-            else:
-                text = " / ".join(messages)
+            text = " / ".join(messages)
         self.progressMessage.setText(text)
         if transport_activity is not None:
             self.transportActivity.setText(transport_activity)
@@ -636,9 +637,9 @@ class SubProcessWidget(QtGui.QWidget):
         for line in data.splitlines():
             if line.startswith(SUB_PROGRESS):
                 try:
-                    # map(ensure_unicode, ...) to ensure that we have unicode after bdecode
-                    progress, transport_activity, messages = map(ensure_unicode,
-                        bencode.bdecode(line[len(SUB_PROGRESS):]))
+                    progress, transport_activity, task_info = bencode.bdecode(
+                        line[len(SUB_PROGRESS):])
+                    messages = [b.decode("utf-8") for b in task_info]
                 except ValueError, e:
                     # we got malformed data from qsubprocess (bencode failed to decode)
                     # so just show it in the status console
@@ -673,6 +674,20 @@ class SubProcessWidget(QtGui.QWidget):
                 
                 data = (button == QtGui.QMessageBox.Yes)
                 self.process.write(SUB_GETBOOL + bencode.bencode(data) + "\n")
+            elif line.startswith(SUB_CHOOSE):
+                msg, choices, default = bdecode_choose_args(line[len(SUB_CHOOSE):])
+                mbox = QtGui.QMessageBox(parent=self)
+                mbox.setText(msg)
+                mbox.setIcon(QtGui.QMessageBox.Question)
+                choices = choices.split('\n')
+                index = 0
+                for c in choices:
+                    button = mbox.addButton(c, QtGui.QMessageBox.AcceptRole)
+                    if index == default:
+                        mbox.setDefaultButton(button)
+                    index += 1
+                index = mbox.exec_()
+                self.process.write(SUB_CHOOSE + bencode.bencode(index) + "\n")
             elif line.startswith(SUB_ERROR):
                 self.error_class, self.error_data = bdecode_exception_instance(
                     line[len(SUB_ERROR):])
@@ -788,20 +803,22 @@ class SubprocessProgressView (TextProgressView):
 
     def _repaint(self):
         if self._last_task:
-            task_msg = self._format_task(self._last_task)
+            # Since bzr 2.2 _format_task returns a 2-tuple of unicode
+            text, counter = self._format_task(self._last_task)
+            task_info = (text.encode("utf-8"), counter.encode("utf-8"))
             progress_frac = self._last_task._overall_completion_fraction()
             if progress_frac is not None:
                 progress = int(progress_frac * 1000000)
             else:
                 progress = 1
         else:
-            task_msg = ''
+            task_info = ()
             progress = 0
 
         trans = self._last_transport_msg
 
         self._term_file.write(
-            SUB_PROGRESS + bencode.bencode((progress, trans, task_msg)) + '\n')
+            SUB_PROGRESS + bencode.bencode((progress, trans, task_info)) + '\n')
         self._term_file.flush()
 
     def clear(self):
@@ -831,6 +848,15 @@ class SubprocessUIFactory(TextUIFactory):
             return bencode.bdecode(line[len(name):].rstrip('\r\n'))
         raise Exception("Did not recive a answer from the main process.")
     
+    def _choose_from_main(self, msg, choices, default):
+        name = SUB_CHOOSE
+        self.stdout.write(name + bencode_choose_args(msg, choices, default) + '\n')
+        self.stdout.flush()
+        line = self.stdin.readline()
+        if line.startswith(name):
+            return bencode.bdecode(line[len(name):].rstrip('\r\n'))
+        raise Exception("Did not recive a answer from the main process.")
+    
     def get_password(self, prompt='', **kwargs):
         prompt = prompt % kwargs
         passwd, accepted = self._get_answer_from_main(SUB_GETPASS, prompt)
@@ -850,6 +876,12 @@ class SubprocessUIFactory(TextUIFactory):
     def get_boolean(self, prompt):
         return self._get_answer_from_main(SUB_GETBOOL, prompt+'?')
 
+    def choose(self, msg, choices, default=None):
+        if default is None:
+            default = -1
+        index = self._choose_from_main(msg, choices, default)
+        return index
+
 
 # [bialix 2010/02/04] body of cmd_qsubprocess has moved from commands.py
 # to see annotation of cmd_qsubprocess before move use:
@@ -956,6 +988,20 @@ def bencode_prompt(arg):
 def bdecode_prompt(s):
     return bencode.bdecode(s).decode('unicode-escape')
 
+def bencode_choose_args(msg, choices, default):
+    if default is None:
+        default = -1
+    return bencode.bencode([
+        msg.encode('unicode-escape'),
+        choices.encode('unicode-escape'),
+        default
+    ])
+
+def bdecode_choose_args(s):
+    msg, choices, default = bencode.bdecode(s)
+    msg = msg.decode('unicode-escape')
+    choices = choices.decode('unicode-escape')
+    return msg, choices, default
 
 def bencode_exception_instance(e):
     """Serialise the main information about an exception instance with bencode
